#!/bin/gawk -f
# (C) 2002 Vladimir Ivaschenko <vi@maks.net>, licensed under GPL2

function can_read(file,  waste) 
{
    if ((getline waste <file) < 0) {
       return 0
    }
    else {
	close(file)
	return 1
    }
}

function get_proc_wless(ifnum, colnum)
{
	curif=0
        while (getline < PROC_WIRELESS) {
	    if ($0 == "") break
    	    if (!(/Inter/ || /face/)) {
		gsub(":", "", $1)
		if ($1 == "")
		    continue
		curif=curif+1
		ifnames[curif]=$1
		if (curif == ifnum) {
		    rec=$(colnum)
		    gsub(/\./, "", rec)
		    close(PROC_WIRELESS)
        	    return rec
		}
	    }
	}
	close(PROC_WIRELESS)
}

function parse_ethers()
{
	curif=0
        while (getline < "/etc/ethers") {
	    if ($0 == "") break
	    if (match($0, /^([:[:alnum:]]+) ([[:alnum:]]+)/)) {
		macaddr=gensub(/^([:[:alnum:]]+) ([[:alnum:]]+)/, "\\1", "g", $0)
		alias=gensub(/^([:[:alnum:]]+) ([[:alnum:]]+)/, "\\2", "g", $0)
		ethers[toupper(macaddr)]=alias
	    }
	}
}

function get_sta_list(ifnum, ifname)
{
    if (!can_read(PROC_PRISM "/" ifname "/stats"))
	return 0
	
    while ("/bin/ls -w1 " PROC_PRISM "/" ifname "/" | getline STA) {
	if (STA == "") break
	if (!(match(STA, /..\:..\:..\:..\:..\:../))) {
	    continue
	}
	stacount[ifnum]++
    }
    close ("/bin/ls -w1 " PROC_PRISM "/" ifname "/")
}

function get_sta_info(ifname, stan, stavarn)
{
    if (!can_read(PROC_PRISM "/" ifname "/stats"))
	return 0

    parse_ethers()

    cur_stan=0
    
    while ("/bin/ls -w1 " PROC_PRISM "/" ifname "/" | getline STA) {
	if (STA == "") break

	if (!(match(STA, /..\:..\:..\:..\:..\:../))) {
	    continue
	}
	
	STAFILE = PROC_PRISM "/" ifname "/" STA
	
	delete stainfo
	cur_stan++;
	if (cur_stan != stan)
	    continue;
	    
	while (getline < STAFILE) {
	    if ($0 == "") break
	    if (match($0, /^([_\[\]=[:alnum:]])+=(.*)/)) {
		varname=gensub(/^([_\[\][:alnum:]]+)=(.*)/, "\\1", "g", $0)
		varvalue=gensub(/^([_\[\][:alnum:]]+)=(.*)/, "\\2", "g", $0)
		stainfo[varname]=varvalue
	    }
	    if (match($0, /^last_rx: silence=([[:digit:]]+) signal=([[:digit:]]+) rate=([[:digit:]]+) flow=([[:digit:]]+)/)) {
		stainfo["lrx_silence"]=gensub(/^last_rx: silence=([[:digit:]]+) signal=([[:digit:]]+) rate=([[:digit:]]+) flow=([[:digit:]]+)/, "\\1", "g", $0)
		stainfo["lrx_signal"]=gensub(/^last_rx: silence=([[:digit:]]+) signal=([[:digit:]]+) rate=([[:digit:]]+) flow=([[:digit:]]+)/, "\\2", "g", $0)
		stainfo["lrx_rate"]=gensub(/^last_rx: silence=([[:digit:]]+) signal=([[:digit:]]+) rate=([[:digit:]]+) flow=([[:digit:]]+)/, "\\3", "g", $0)
	    }
	}
	close(STAFILE)
	if (ethers[toupper(stainfo["STA"])] == "")
	    alias=toupper(stainfo["STA"])
	else
	    alias=ethers[toupper(stainfo["STA"])]

	close ("/bin/ls -w1 " PROC_PRISM "/" ifname "/")

	if (stavarn == 1)
	    return "string\n" toupper(stainfo["STA"])
	else if (stavarn == 2)
	    return "string\n" alias
	else if (stavarn == 3)
	    return "gauge\n" stainfo["lrx_signal"]
	else if (stavarn == 4)
	    return "gauge\n" stainfo["lrx_silence"]
	else if (stavarn == 5)
	    return "gauge\n" stainfo["tx_rate"]
	else if (stavarn == 6)
	    return "gauge\n" stainfo["lrx_rate"]
	else if (stavarn == 7)
	    return "counter\n" stainfo["tx_bytes"]
	else if (stavarn == 8)
	    return "counter\n" stainfo["rx_bytes"]
    }
}

BEGIN {
    PROC_WIRELESS="/proc/net/wireless"
    IWCONFIG="/sbin/iwconfig "
    PROC_PRISM="/proc/net/hostap"
    SNMP_PLACE=".1.3.6.1.4.1.2021.254"
    LAST_FUNC=16
    LAST_STA_VAR=8

    OID=ARGV[2]

    f_id = OID
    sub(SNMP_PLACE ".", "", f_id)
    sub(/\..*/, "", f_id)
    # Force f_id to be treated as a number by awk
    f_id += 0

    base_oid = SNMP_PLACE "." f_id
    params = OID
    sub(base_oid, "", params)
    ifnum = gensub(/\.([[:digit:]]+).*/, "\\1", "g", params)
    sub(/\..*/, "", ifnum)
    ifnum += 0
        
    get_proc_wless(-1, -1)
    ifcount = curif

    if ( f_id == 16 || (ARGV[1] == "-n" && f_id == 15) ) {
        while ("/bin/ls -w1 /tmp/" | getline FNAME) {
    	    if (FNAME == "") break
	    if (match(FNAME, /newname\..*/ )) {
	        old_ifname = gensub(/newname\.(.*)/, "\\1", "g", FNAME)
	        FNAME="/tmp/" FNAME
	        getline new_ifname < FNAME
	        old_ifnames[ifnum]=old_ifname
		get_sta_list(ifnum, old_ifname)
    	    }
	}
    }

    if (f_id == 16) {
	sta_num = gensub(/\.[[:digit:]]+\.([[:digit:]]+)\.[[:digit:]]+$/, "\\1", "g", params)
	stavar_num = gensub(/\.[[:digit:]]+\.[[:digit:]]+\.([[:digit:]]+)$/, "\\1", "g", params)
    }
    
    # Check if we need to find next OID
    if (ARGV[1] == "-n") {

	# AP stats require special treatment
	if (f_id == 16) {

	    if (stavar_num < LAST_STA_VAR) {
		stavar_num++
		foundnext = 1
	    }
	    
	    if (!foundnext && stavar_num >= LAST_STA_VAR) {
		stavar_num = 1
		sta_num++
		foundnext = 1
	    }

	    if ( (!foundnext && sta_num >= stacount[ifnum]) \
	        || sta_num > stacount[ifnum]) {
		sta_num = 1
		stavar_num = 1
		ifnum++
		foundnext = 1
	    }
		
	    OID = SNMP_PLACE "." f_id "." ifnum "." sta_num "." stavar_num
	    foundnext = 1
	}

	if (f_id == 0) {
	    f_id = 1
	    ifnum = 1
	    OID = SNMP_PLACE "." f_id ".1"
	    foundnext = 1
	}
	if (ifnum == 0) {
	    ifnum = 1
	    OID = SNMP_PLACE "." f_id ".1"
	    foundnext = 1
	}	    
	if (!foundnext && ifnum >= ifcount) {
	    f_id++
	    ifnum = 1
	    OID = SNMP_PLACE "." f_id ".1"
	    foundnext = 1
	} 

	if (!foundnext) {
	    ifnum++
	    OID = SNMP_PLACE "." f_id "." ifnum
	    foundnext = 1
	}

	# AP stats require special treatment
	if (f_id == 16 && !sta_num) {
		i=0
		while (i <= ifcount) {
		    if (stacount[i] > 0) {
			ifnum = i
			sta_num = 1
			stavar_num = 1
			break
		    }
		    i++
		}
		if (i > ifcount) {
		    exit 0
		}
		else
		    OID = SNMP_PLACE "." f_id ".1.1.1"
	}

	if (f_id > LAST_FUNC)
	    exit 0
    }

    if (ifnum > ifcount)
	exit 0

    # The first 11 "functions" just give the corresponding column
    # from /proc/net/wireless
    
    if (f_id >= 1 && f_id <= 11) { 
	print OID
	if (f_id == 1)
		print "string"
	else if (f_id <= 5)
		print "gauge"
	else
		print "counter"
	print get_proc_wless(ifnum, f_id)
	exit 0
    }
    
    # Discarded packets (sum)
    if (f_id == 12) {
	print OID
	print "counter"
	print (get_proc_wless(ifnum, 6)+get_proc_wless(ifnum, 7) \
	      +get_proc_wless(ifnum, 8)+get_proc_wless(ifnum, 9) \
	      +get_proc_wless(ifnum, 10))
	exit 0
    }
    
    # Current Bit Rate
    if (f_id == 13) {
	while (IWCONFIG ifnames[ifnum] | getline) {
	    if ($0 == "") break
	    if (match($0, /Bit Rate:([[:digit:]]+)Mb\/s /)) {
		bitrate = gensub(/.*Bit Rate:([[:digit:]]+)Mb\/s .*/, "\\1", "g", $0)
		print OID
		print "gauge"
		print bitrate
		exit 0
	    }
	}
    }
    
    # Access Point
    if (f_id == 14) {
	while (IWCONFIG ifnames[ifnum] | getline) {
	    if ($0 == "") break
	    if (match($0, /Access Point: ([0123456789ABCDEF\:]+)/)) {
		value = gensub(/.*Access Point: ([0123456789ABCDEF\:]+).*/, "\\1", "g", $0)
		print OID
		print "string"
		print value
		exit 0
	    }
	}	    
    }
    
    # ESSID
    if (f_id == 15) {
	while (IWCONFIG ifnames[ifnum] | getline) {
	    if ($0 == "") break
	    if (match($0, /ESSID:([\"[:alnum:]]+) /)) {
		value = gensub(/.*ESSID:([\"[:alnum:]]+) .*/, "\\1", "g", $0)
		gsub("\"", "", value)
		print OID
		print "string"
		print value
		exit 0
	    }
	}	    
    }

    # Access Point statistics
    if (f_id == 16) {
	print OID
	print get_sta_info(old_ifnames[ifnum], sta_num, stavar_num)
    }

}
